/*
 * Copyright © 2012-2013 Mike Gabriel <mike.gabriel@das-netzwerkteam.de>
 * Copyright © 2012 Canonical Ltd.
 *
 * This program is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 3, as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranties of
 * MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
 * PURPOSE.  See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Author: Mike Gabriel <mike.gabriel@das-netzwerkteam.de>
 */

#ifndef _PAM_X2GO_H_
#define _PAM_X2GO_H_

#define PAM_X2GO_PROMPT_GUESTLOGIN	"login:"
#define PAM_X2GO_PROMPT_USER		"remote login:"
#define PAM_X2GO_PROMPT_HOST		"remote host:"
#define PAM_X2GO_PROMPT_COMMAND		"remote command:"
#define PAM_X2GO_PROMPT_PASSWORD	"password:"

#endif //_PAM_X2GO_H_
